/*
    TCFbERowColumnCachedTable.cpp    November 29, 2004.

    Copyright (C) 2003-2005 CFbE Research Group,
    Software Engineering Laboratory,
    Graduate School of Information Science,
    Nara Institute of Science and Technology,
    All rights reserved.

    This program is free software; you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2, (at your option) or
    any later version.

    This program is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with GNU Emacs; see the file COPYING.  If not, write to the
    Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
*/
//---------------------------------------------------------------------------
#pragma hdrstop

#include "TCFbERowColumnCachedTable.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
//---------------------------------------------------------------------------
// コンストラクタ（空キャッシュを作成
__fastcall TCFbERowColumnCachedTable::TCFbERowColumnCachedTable(void)
    : TCFbECachedTable()
{
}

//---------------------------------------------------------------------------
// 指定された行列を備えた空テーブルを作成
__fastcall TCFbERowColumnCachedTable::TCFbERowColumnCachedTable(TStringList* RowLabelList, TStringList* ColumnLabelList)
    : TCFbECachedTable(RowLabelList, ColumnLabelList)
{
}

//---------------------------------------------------------------------------
// コンストラクタ
__fastcall TCFbERowColumnCachedTable::TCFbERowColumnCachedTable(TStringList* SourceCsvStringList)
    : TCFbECachedTable(SourceCsvStringList)
{
}

//---------------------------------------------------------------------------
// デストラクタ
__fastcall TCFbERowColumnCachedTable::~TCFbERowColumnCachedTable()
{
}

//---------------------------------------------------------------------------
// 以下，protected method
//---------------------------------------------------------------------------
// 親キャッシュの行が追加された
int __fastcall  TCFbERowColumnCachedTable::ParentAddRow(const char* RowLabel)
{
    return this->AddRow(RowLabel);
}

//---------------------------------------------------------------------------
// 親キャッシュの列が追加された
int __fastcall  TCFbERowColumnCachedTable::ParentAddColumn(const char* ColumnLabel)
{
    return this->AddColumn(ColumnLabel);
}

//---------------------------------------------------------------------------
// 親キャッシュの行が削除された
void __fastcall TCFbERowColumnCachedTable::ParentDeleteRowByIndex(int TargetRowIndex)
{
    this->DeleteRowByIndex(TargetRowIndex);
}

//---------------------------------------------------------------------------
// 親キャッシュの列が削除された
void __fastcall TCFbERowColumnCachedTable::ParentDeleteColumnByIndex(int TargetColumnIndex)
{
    this->DeleteColumnByIndex(TargetColumnIndex);
}

//---------------------------------------------------------------------------
// 自分自身のクローンを作成する
TCFbECachedTable* __fastcall TCFbERowColumnCachedTable::CreateClone(void)
{
    TCFbERowColumnCachedTable* Clone = new TCFbERowColumnCachedTable();
    Clone->Assign(this);
    return Clone;
}

//---------------------------------------------------------------------------
// 自分自身の行と列が入れ替わったクローンを作成する
TCFbECachedTable* __fastcall TCFbERowColumnCachedTable::CreateRotatedClone(void)
{
    TCFbERowColumnCachedTable* Clone = new TCFbERowColumnCachedTable();
    Clone->RotatedAssign(this);
    return Clone;
}

//---------------------------------------------------------------------------

